package car.controllers;

import java.io.IOException;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import car.models.com.data.CarCond;
import car.models.com.data.CarContainer;
import car.models.com.data.PriceRange;
import car.models.db.DBCar;

/**
 * メインサーブレット
 * @author
 *
 */
public class MainServlet extends HttpServlet {

    @Override
    public void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        doPost(req, resp);
    }

    @Override
    public void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        String nextPage = "/view/html/Main.html";

        // 文字化け回避のため、UTF-8にエンコードする
        req.setCharacterEncoding("UTF-8");

        // セッションオブジェクトを取得する
        HttpSession session = req.getSession(true);

        // 画面から受け取った車情報コンテナを生成する
        CarContainer input = new CarContainer();
        // 画面へ渡す車情報コンテナを生成する
        CarContainer output = new CarContainer();

        // 画面から受け取ったリクエスト名を取得する
        String reqName = req.getParameter("request");

        // リクエスト名を判定する
        if ("RQSearch".equals(reqName)) {
            // 検索リクエストの場合

            // 車情報コンテナに検索条件をセットする
            setCarCond(req, input);

            // DB処理
            DBCar dbCar = new DBCar();
            dbCar.setInputValue(input);
            // 検索を行う
            if(dbCar.select()) {
                // 検索が正常に行われた場合、検索結果を取得する
                output = dbCar.getOutputValue();
            }
            nextPage = "/view/jsp/CarList.jsp";

        } else if ("RQRegister".equals(reqName)) {
            // 登録リクエストの場合

        } else if ("RQUpdate".equals(reqName)) {
            // 更新リクエストの場合
        }

        // 次の画面に渡す値をセットする
        session.setAttribute("value", output);

        // 次のページへ移動する
        RequestDispatcher rd = req.getRequestDispatcher(nextPage);
        rd.forward(req, resp);
    }

    /**
     * 車の検索条件をセットする
     * @param req [in] リクエストオブジェクト
     * @param car [ref] 車情報コンテナ
     */
    private void setCarCond(HttpServletRequest req, CarContainer car) {
        CarCond cond = new CarCond();

        // 「条件：メーカー」の取得およびセット
        String[] condMaker = req.getParameterValues("chkCondMaker");
        if (condMaker != null) {
            for (int i = 0; i < condMaker.length; i++) {
                if ("ALL".equals(condMaker[i])) {
                    // 条件に'ALL'を含む場合、条件は設定せず、処理を抜ける
                    cond.getCondMaker().clear();
                    break;
                } else {
                    cond.getCondMaker().add(condMaker[i]);
                }
            }
        }

        // 「条件：カテゴリー」の取得およびセット
        String[] condCategory = req.getParameterValues("chkCondCategory");
        if (condCategory != null) {
            for (int i = 0; i < condCategory.length; i++) {
                if ("ALL".equals(condCategory[i])) {
                    cond.getCondCategory().clear();
                    break;
                } else {
                    cond.getCondCategory().add(condCategory[i]);
                }
            }
        }

        // 「条件：価格帯」の取得およびセット
        String[] condPrice = req.getParameterValues("chkCondPrice");
        if (condPrice != null) {
            for (int i = 0; i < condPrice.length; i++) {
                if ("ALL".equals(condPrice[i])) {
                    cond.getCondPrice().clear();
                    break;
                } else {
                    cond.getCondPrice().add(convertPrice(condPrice[i]));
                }
            }
        }

        car.setCarCond(cond);
    }

    /**
     * 価格帯情報を変換する
     * @param range 価格帯情報(文字列)
     * @return 価格帯情報(PriceRange型)
     */
    private PriceRange convertPrice(String range) {
        PriceRange pr = null;

        // 下限値と上限値を分割する
        String[] lu = range.split("-");
        if (lu.length != 2) {
            // 下限、上限の2値が取得できない場合は、失敗とみなす
            return pr;
        }
        // PriceRange型として生成する
        pr = new PriceRange(lu[0],lu[1]);

        return pr;
    }
}
