package car.models.db;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import car.models.com.data.CarCond;
import car.models.com.data.CarContainer;
import car.models.com.data.CarInfo;
import car.models.com.data.PriceRange;
import car.models.com.util.CommonUtility;

/**
 * Carテーブル操作クラス
 * @author
 *
 */
public class DBCar extends DBCommon {
    private static final String DBTBL_NAME = "car";

    private CarContainer InputValue;
    private CarContainer OutputValue;

    public CarContainer getInputValue() {
        return InputValue;
    }

    public void setInputValue(CarContainer inputValue) {
        InputValue = inputValue;
    }

    public CarContainer getOutputValue() {
        return OutputValue;
    }

    public void setOutputValue(CarContainer outputValue) {
        OutputValue = outputValue;
    }

    /**
     * SQL:selectを実行する
     * @return true: 成功 / false: 失敗
     */
    public boolean select() {
        try {
            // DB接続オブジェクトを取得する
            Connection conn = getConnection();

            StringBuffer sql = new StringBuffer();

            // select文を生成する
            sql.append("SELECT ");
            sql.append("    * ");
            sql.append("FROM ");
            sql.append("    " + DBTBL_NAME);

            // where条件を設定する
            // ※検索条件がなければ設定しない
            if (InputValue != null && InputValue.getCarCond() != null) {
                StringBuffer whereSql = new StringBuffer();
                boolean existCond = false;

                CarCond cond = InputValue.getCarCond();

                // 条件：メーカーを設定する
                String condMaker = combineCond(cond.getCondMaker(), ",", true);
                if (!CommonUtility.isNullOrEmpty(condMaker)) {
                    whereSql.append("maker in(").append(condMaker).append(")");
                    existCond = true;
                }

                // 条件：カテゴリーを設定する
                String condCategory = combineCond(cond.getCondCategory(), ",", true);
                if (!CommonUtility.isNullOrEmpty(condCategory)) {
                    if (existCond) {
                        // 条件が既に存在する場合、ANDで連結する
                        whereSql.append(" AND ");
                    }

                    whereSql.append("category in(").append(condCategory).append(")");
                    existCond = true;
                }

                // 条件：価格帯を設定する
                ArrayList<String> priceRangeList = new ArrayList<String>();
                // 上限、下限の条件を設定する
                for (int i = 0; i < cond.getCondPrice().size(); i++) {
                    PriceRange pr = cond.getCondPrice().get(i);
                    StringBuffer sbRange = new StringBuffer();
                    sbRange.append("( ");
                    sbRange.append(pr.getLowerLimit()).append(" <= price");
                    sbRange.append(" AND ");
                    sbRange.append("price <= ").append(pr.getUpperLimit());
                    sbRange.append(")");
                    priceRangeList.add(sbRange.toString());
                }
                if (priceRangeList.size() > 0) {
                    String condPrice = combineCond(priceRangeList, "OR", false);
                    if (existCond) {
                        whereSql.append(" AND ");
                    }

                    whereSql.append("(").append(condPrice).append(")");
                }

                if (!CommonUtility.isNullOrEmpty(whereSql.toString())) {
                    // 条件が設定されていれば、where条件として連結する
                    sql.append(" where ").append(whereSql.toString());
                }
            }

            // SQL文の発行
            PreparedStatement prepStmt = conn.prepareStatement(sql.toString());
            ResultSet rs = prepStmt.executeQuery();

            OutputValue = new CarContainer();

            // selectで取得したデータを車情報コンテナにセットする
            while (rs.next()) {
                CarInfo carInfo = new CarInfo();
                carInfo.setCarName(rs.getString("car_name"));
                carInfo.setMaker(rs.getString("maker"));
                carInfo.setCategory(rs.getString("category"));
                carInfo.setPrice(rs.getInt("price"));
                OutputValue.getCarInfoList().add(carInfo);
            }

            conn.close();
        }
        catch (SQLException e) {
            return false;
        }

        return true;
    }

    /**
     * SQL:insertを実行する
     * @return true: 成功 / false: 失敗
     */
    public boolean insert() {
        return true;
    }

    /**
     * SQL:updateを実行する
     * @return true: 成功 / false: 失敗
     */
    public boolean update() {
        return true;
    }
}
