package test.servlet;

import static org.junit.Assert.*;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpSession;

import org.junit.BeforeClass;
import org.junit.Test;
import org.springframework.mock.web.MockHttpServletRequest;
import org.springframework.mock.web.MockHttpServletResponse;

import test.com.data.SampleContainer;
import test.com.util.COMDataDump;

public class MainServletTest {
    private static List PackageList;

    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
        PackageList = new ArrayList<String>();
        PackageList.add("test.com.data");
    }

    @Test
    public void test() {
        assertTrue(doPostTest());
    }

    public boolean doPostTest() {
        // 1. サーブレットの生成
        SampleServlet servlet = new SampleServlet();

        // 2. リクエスト、レスポンスの生成
        MockHttpServletRequest req = new MockHttpServletRequest();
        MockHttpServletResponse res = new MockHttpServletResponse();

        // 3. リクエスト通信方式をPOSTに設定
        // ※ 画面でいう<form method="post">の代わりになるもの
        req.setMethod("POST");

        // 4. リクエストパラメータを設定
        // ※ 画面でいう<input name="xxx" value="xxx"/>の代わりになるもの
        req.setParameter("request", "RQSet");
        req.setParameter("name", "鈴木");
        req.setParameter("category", "categoryA");
        req.setParameter("result", "100000");

        // 5. サーブレットの実行
        try {
            servlet.init();
            servlet.doPost(req, res);
            servlet.destroy();
        }
        catch (Exception e) {
            e.printStackTrace();
            return false;
        }

        // 6. サーブレット内でセッションに設定したコンテナ情報の取得
        HttpSession session = req.getSession();
        SampleContainer container = (SampleContainer)session.getAttribute("value");

        COMDataDump dump = new COMDataDump();
        try {
            dump.setTargetCategoryFromPackage(PackageList);
            Field field = SampleContainer.class.getDeclaredField("InfoList");
            dump.dumpData(container, field);
        }
        catch (Exception e) {
            e.printStackTrace();
            return false;
        }

        // 7. 次に表示するページの取得
        String nextPage = res.getForwardedUrl();
        System.out.println("Next Page : " + nextPage);

        return true;
    }

}
